#include "converterfunctions.h"
#include "converter.h"

void convertToGCol(int **puzzle, int order, int puzLen, bool **adjacent, int numNodes, int &edgesDueToFilledCells)
{
	int r,r1,r2,c,c1,c2;

	//First, make all entries in the adjaceny matrix false
	for(r=0; r<numNodes; r++) for(c=0; c<numNodes; c++) adjacent[r][c]=false;

	//Now make all nodes adjacent to themselves
	for(r=0; r<numNodes; r++) adjacent[r][r]=true;

	//Now add edges between each all nodes that represent cells on the same row
	for(r=0; r<puzLen; r++){
		for(c=0; c<puzLen-1; c++){
			for(c2=c+1; c2<puzLen; c2++){
				//	cout<<"adding edges between nodes "<<r*puzLen+c<<" and "<<r*puzLen+c2<<endl;
				adjacent[r*puzLen+c][r*puzLen+c2] = true;
				adjacent[r*puzLen+c2][r*puzLen+c] = true;
			}
		}
	}

	//Now add edges between each all nodes that represent cells in the same column
	for(c=0; c<puzLen; c++){
		for(r=0; r<puzLen-1; r++){
			for(r2=r+1; r2<puzLen; r2++){
				//	cout<<"adding edges between nodes "<<r*puzLen+c<<" and "<<r2*puzLen+c<<endl;
				adjacent[r*puzLen+c][r2*puzLen+c] = true;
				adjacent[r2*puzLen+c][r*puzLen+c] = true;
			}
		}
	}
	
	//Now add edges between each all nodes that represent cells in the same box
	for(r=0;r<order;r++){
		for(c=0;c<order;c++){
			//Now go through each pair of cells in box [r, c]
			for(r1=r*order; r1<(r+1)*order; r1++){
				for(c1=c*order; c1<(c+1)*order; c1++){
					for(r2=r*order; r2<(r+1)*order; r2++){
						for(c2=c*order; c2<(c+1)*order; c2++){
							adjacent[r1*puzLen+c1][r2*puzLen+c2] = true;
							adjacent[r2*puzLen+c2][r1*puzLen+c1] = true;
						}
					}
				}
			}
		}
	}

	//Finally, add adges between any two nodes that have been assigned different colours
	//(i.e. add edges between pairs of filled cells that don't contain the same values)
	int i, j;
	for(i=0; i<numNodes-1; i++){
		r=i/puzLen;
		c=i%puzLen;
		if(puzzle[r][c]>0){
			//go through the other cells in the grid and find ones with different filled values to puzzle[r][c]
			for(j=i; j<numNodes; j++){
				r1=j/puzLen;
				c1=j%puzLen;
				if(puzzle[r1][c1]>0 && (puzzle[r1][c1] != puzzle[r][c])){
					//add an edge
					edgesDueToFilledCells++;
					adjacent[i][j] = true;
					adjacent[j][i] = true;
				}
			}
		}
	}		
}


//------------------------------------------------------------------------------------------------
int calcNumEdges(bool **adjacent, int numNodes)
{
	int r, c;
	int count = 0;
	for(r=0; r<numNodes; r++){
		for(c=0; c<r; c++){
			if(adjacent[r][c])count++;
		}
	}
	return count;
}
	

//------------------------------------------------------------------------------------------------
void makeOutput(char *filename, ofstream &outStream, int numNodes, 
				int  **puzzle, bool **adjacent, int numEdges, char *infileName, int order, int edgesDueToFilledCells)
{
	//PRINT OUT AL THE DETAILS OF THE GRAPH COLOURING PROBLEM TO THE OUTPUT FILE
	int r, c;
	outStream<<"c FILE "<<filename<<endl;
	outStream<<"c\n";
	outStream<<"c\n";
	outStream<<"c SOURCE This file was made using the order-"<<order<<" sudoku grid stored in the file "<<infileName<<endl;
	outStream<<"c        The sudoku grid needs to be filled with numbers 1 to "<<order*order<<endl;
	outStream<<"c        The grid is of dimensions "<<order*order<<" by "<<order*order<<endl;
	outStream<<"c        There are "<<order*order<<" "<<order<<"x"<<order<<" boxes"<<endl;
	outStream<<"c\n";
	outStream<<"c\n";
	outStream<<"c THE GRID WAS AS FOLLOWS:\n";
	outStream<<"c\n";
	outStream<<"c\n";
	for(r=0;r<order*order; r++){
		outStream<<"c\t";
		for(c=0; c<order*order; c++){
			if(puzzle[r][c] == 0) 
				outStream<<"[-]\t";
			else
				outStream<<"["<<puzzle[r][c]<<"]\t";
		}
		outStream<<endl;
	}
	outStream<<"c\n";
	outStream<<"c\n";
	outStream<<"c DESCRIPTION This graph colouring problem was generated from a \n";
	outStream<<"c    sudoku grid using the converter designed by Rhydian Lewis\n";
	outStream<<"c    If you notice any problems with this program then please contact\n";
	outStream<<"c    Rhydian Lewis by emailing LewisR9@cf.ac.uk\n";
	outStream<<"c\n";
	outStream<<"c\n";
	outStream<<"c GRAPH STATS (assuming the input is valid and contains at least one solution)\n";
	outStream<<"c Maximum Clique Size                 = "<<order*order<<endl;
	outStream<<"c Number of Edges                     = "<<numEdges<<endl;
	outStream<<"c Number of Edges Due to filled cells = "<<edgesDueToFilledCells<<endl;
	outStream<<"c Number of Nodes                     = "<<numNodes<<endl;
	outStream<<"c Optimum number of colours           = "<<order*order<<endl;
	outStream<<"c\n";
	outStream<<"c\n";
	outStream<<"p edge "<<numNodes<<" "<<numEdges<<endl;
	for(r=0; r<numNodes; r++){
		for(c=0; c<r; c++){
			if(adjacent[r][c])outStream<<"e "<<r+1<<' '<<c+1<<'\n';
		}
	}
}

